@@ -7,7 +7,7 @@ abstracting away java lucene primitives.
 
 import warnings, lucene
 if lucene.VERSION <= '2.4.1':
-    warnings.warn('Support for lucene 2.4 will be removed in a future release.', DeprecationWarning)
+    warnings.warn('Support for lucene 2.4 will be removed in the next release.', DeprecationWarning)
 
 from .queries import Query, Filter
 from .documents import Document, Field, FormatField, PrefixField, NestedField, DateTimeField
