@@ -4,6 +4,7 @@ Wrappers for lucene Fields and Documents.
 
 import itertools
 import datetime
+import collections
 import lucene
 from .queries import Query
 
@@ -55,8 +56,8 @@ class PrefixField(Field):
     
     :param start, stop, step: optional slice parameters of the prefix depths (not indices)
     """
-    def __init__(self, name, start=1, stop=None, step=1, store=False, index=True, termvector=False):
-        Field.__init__(self, name, store, index, termvector)
+    def __init__(self, name, start=1, stop=None, step=1, index=True, **kwargs):
+        Field.__init__(self, name, index=index, **kwargs)
         self.depths = slice(start, stop, step)
     def split(self, text):
         "Return immutable sequence of words from name or value."
@@ -66,7 +67,7 @@ class PrefixField(Field):
         return words
     def getname(self, depth):
         "Return prefix field name for given depth."
-        return '{0}[:{1:n}]'.format(self.name, depth)
+        return '{0}[:{1:d}]'.format(self.name, depth)
     def indices(self, depth):
         "Return range of valid depth indices."
         return xrange(*self.depths.indices(depth + self.depths.step))
@@ -153,7 +154,7 @@ class DateTimeField(PrefixField):
         dates = (map(float, self.split(str(date))) for date in (start, stop))
         items = []
         for dates in self._range(*dates):
-            items.append(tuple(self.join(map('{0:02n}'.format, date)) for date in dates))
+            items.append(tuple(self.join(map('{0:02g}'.format, date)) for date in dates))
         queries = [PrefixField.range(self, str(start), items[0][1], lower=lower)]
         queries += [PrefixField.range(self, *item) for item in items[1:-1]]
         queries.append(PrefixField.range(self, items[-1][0], str(stop), upper=upper))
@@ -185,13 +186,10 @@ class DateTimeField(PrefixField):
 class Document(object):
     """Delegated lucene Document.
     Provides mapping interface of field names to values, but duplicate field names are allowed.
-    
-    :param doc: optional lucene Document
     """
     def __init__(self, doc=None):
         self.doc = lucene.Document() if doc is None else doc
     def add(self, name, value, cls=Field, **params):
-        "Add field to document with given parameters."
         for field in cls(name, **params).items(value):
             self.doc.add(field)
     def fields(self):
@@ -256,18 +254,20 @@ class Hits(object):
     :param ids: ordered doc ids
     :param scores: ordered doc scores
     :param count: total number of hits
+    :param fields: optional field selectors
     """
-    def __init__(self, searcher, ids, scores, count=0):
+    def __init__(self, searcher, ids, scores, count=0, fields=None):
         self.searcher = searcher
         self.ids, self.scores = ids, scores
         self.count = count or len(self)
+        self.fields = lucene.MapFieldSelector(fields) if isinstance(fields, collections.Iterable) else fields
     def __len__(self):
         return len(self.ids)
     def __getitem__(self, index):
         id, score = self.ids[index], self.scores[index]
         if isinstance(index, slice):
-            return type(self)(self.searcher, id, score, self.count)
-        return Hit(self.searcher.doc(id), id, score)
+            return type(self)(self.searcher, id, score, self.count, self.fields)
+        return Hit(self.searcher.doc(id, self.fields), id, score)
     def items(self):
         "Generate zipped ids and scores."
         return itertools.izip(self.ids, self.scores)
