@@ -9,7 +9,7 @@ import contextlib
 import abc, collections
 import warnings
 import lucene
-from .queries import Query, HitCollector, Filter, Highlighter
+from .queries import Query, HitCollector, Filter, Highlighter, SpellChecker, SpellParser
 from .documents import Field, Document, Hits
 
 class Atomic(object):
@@ -109,19 +109,23 @@ class Analyzer(lucene.PythonAnalyzer):
     def tokens(self, text, field=None):
         "Return lucene TokenStream from text."
         return self.tokenStream(field, lucene.StringReader(text))
-    def parse(self, query, field='', op='', version='current', **attrs):
+    def parse(self, query, field='', op='', version=lucene.VERSION[:3], parser=None, **attrs):
         """Return parsed lucene Query.
         
         :param query: query string
         :param field: default query field name, sequence of names, or (in lucene 2.9 or higher) boost mapping
         :param op: default query operator ('or', 'and')
         :param version: lucene Version string, leave blank for deprecated constructor
+        :param parser: custom PythonQueryParser class
         :param attrs: additional attributes to set on the parser
         """
         # parsers aren't thread-safe (nor slow), so create one each time
-        args = []
-        if version and lucene.VERSION >= '2.9.1':
-            args += getattr(lucene.Version, 'LUCENE_' + version.replace('.', '').upper()),
+        try:
+            version = getattr(lucene.Version, 'LUCENE_' + version.replace('.', ''))
+        except AttributeError:
+            args = []
+        else:
+            args = [] if lucene.VERSION.startswith('2.9') and parser is not None else [version]
         if isinstance(field, collections.Mapping):
             boosts = lucene.HashMap()
             for key in field:
@@ -129,14 +133,18 @@ class Analyzer(lucene.PythonAnalyzer):
             args += list(field), self, boosts
         else:
             args += field, self
-        parser = (lucene.QueryParser if isinstance(field, basestring) else lucene.MultiFieldQueryParser)(*args)
+        parser = (parser or lucene.QueryParser if isinstance(field, basestring) else lucene.MultiFieldQueryParser)(*args)
         if op:
             parser.defaultOperator = getattr(lucene.QueryParser.Operator, op.upper())
         for name, value in attrs.items():
             setattr(parser, name, value)
         if isinstance(parser, lucene.MultiFieldQueryParser):
             return parser.parse(parser, query) # bug in method binding
-        return parser.parse(query)
+        try:
+            return parser.parse(query)
+        finally:
+            if isinstance(parser, lucene.PythonQueryParser):
+                parser.finalize()
 
 class IndexReader(object):
     """Delegated lucene IndexReader, with a mapping interface of ids to document objects.
@@ -231,16 +239,25 @@ class IndexReader(object):
         bases = getattr(lucene, 'Python{0}Parser'.format(type)),
         parser = object.__class__('', bases, {'parse' + type: staticmethod(parser)})
         return method(self.indexReader, name, parser())
-    def spans(self, query, positions=False):
+    def spans(self, query, positions=False, payloads=False):
         """Generate docs with occurrence counts for a span query.
         
         :param query: lucene SpanQuery
         :param positions: optionally include slice positions instead of counts
+        :param payloads: optionally only include slice positions with payloads
         """
         spans = query.getSpans(self.indexReader)
         spans = itertools.takewhile(lucene.Spans.next, itertools.repeat(spans))
         for doc, spans in itertools.groupby(spans, key=lucene.Spans.doc):
-            if positions:
+            if payloads:
+                items = []
+                for span in spans:
+                    if span.payloadAvailable:
+                        payloads = map(lucene.JArray_byte.cast_, span.payload)
+                        payloads = [data.string_ if hasattr(data, 'string_') else ''.join(data) for data in payloads]
+                        items.append((span.start(), span.end(), payloads))
+                yield doc, items
+            elif positions:
                 yield doc, [(span.start(), span.end()) for span in spans]
             else:
                 yield doc, sum(1 for span in spans)
@@ -282,6 +299,7 @@ class Searcher(object):
             self.shared.add(analyzer)
         self.analyzer = analyzer
         self.filters = {}
+        self.spellcheckers = {}
     def __del__(self):
         if hash(self):
             self.close()
@@ -295,23 +313,37 @@ class Searcher(object):
                     directory = lucene.FSDirectory.getDirectory(directory)
                 self.shared.add(directory)
             yield directory
-    def reopen(self):
-        "Return current Searcher, only creating a new one if necessary."
+    def reopen(self, filters=False, spellcheckers=False):
+        """Return current Searcher, only creating a new one if necessary.
+        
+        :param filters: refresh cached facet :attr:`filters`
+        :param spellcheckers: refresh cached :attr:`spellcheckers`
+        """
         reader = self.indexReader.reopen()
         if reader == self.indexReader:
             return self
         other = type(self)(reader, self.analyzer)
         other.shared, other.owned = self.shared, closing([reader])
+        if filters:
+            other.facets([], *self.filters)
+        if spellcheckers:
+            for field in self.spellcheckers:
+                other.suggest(field, '')
         return other
     def __getitem__(self, id):
         return Document(self.doc(id))
-    def parse(self, query, *args, **kwargs):
+    def get(self, id, *fields):
+        "Return `Document`_ with only selected fields loaded."
+        return Document(self.doc(id, lucene.MapFieldSelector(fields)))
+    def parse(self, query, spellcheck=False, **kwargs):
         if isinstance(query, lucene.Query):
             return query
-        return Analyzer.parse.im_func(self.analyzer, query, *args, **kwargs)
+        if spellcheck:
+            kwargs['parser'], kwargs['searcher'] = SpellParser, self
+        return Analyzer.parse.im_func(self.analyzer, query, **kwargs)
     def highlighter(self, query, span=True, formatter=None, encoder=None, field=None):
         "Return `Highlighter`_ specific to the searcher's analyzer and index."
-        query = self.parse(query, field or '')
+        query = self.parse(query, field=field or '')
         return Highlighter(query, self.analyzer, span, formatter, encoder, field, (field and self.indexReader))
     def highlight(self, query, doc, count=1, span=True, formatter=None, encoder=None, field=None, **attrs):
         """Return highlighted text fragments which match the query, using internal :meth:`highlighter`.
@@ -353,15 +385,15 @@ class Searcher(object):
         if count is None:
             collector = HitCollector()
             super(Searcher, self).search(query, filter, collector)
-            return Hits(self, *collector.sorted(key=sort, reverse=reverse))
-        if sort is None:
-            topdocs = super(Searcher, self).search(query, filter, count)
-        else:
-            if isinstance(sort, basestring):
-                sort = lucene.SortField(sort, lucene.SortField.STRING, reverse)
-            if not isinstance(sort, lucene.Sort):
-                sort = lucene.Sort(sort)
-            topdocs = super(Searcher, self).search(query, filter, count, sort)
+            try:
+                return Hits(self, *collector.sorted(key=sort, reverse=reverse))
+            finally:
+                collector.finalize()
+        if isinstance(sort, basestring):
+            sort = lucene.SortField(sort, lucene.SortField.STRING, reverse)
+        if not isinstance(sort, (lucene.Sort, type(None))):
+            sort = lucene.Sort(sort)
+        topdocs = super(Searcher, self).search(query, filter, count, *([sort] * bool(sort)))
         scoredocs = list(topdocs.scoreDocs)
         ids, scores = (map(operator.attrgetter(name), scoredocs) for name in ('doc', 'score'))
         return Hits(self, ids, scores, topdocs.totalHits)
@@ -388,6 +420,29 @@ class Searcher(object):
                     filters[value] = Query.term(name, value).filter()
                 counts[name][value] = Filter.overlap(ids, filters[value], self.indexReader)
         return dict(counts)
+    def suggest(self, field, prefix, count=None):
+        "Return ordered suggested words for prefix."
+        if field not in self.spellcheckers:
+            self.spellcheckers[field] = SpellChecker(self.terms(field, counts=True))
+        return self.spellcheckers[field].suggest(prefix, count)
+    def correct(self, field, text, distance=2, minSimilarity=0.5):
+        """Generate potential words ordered by increasing edit distance and decreasing frequency.
+        For optimal performance only iterate the required slice size of corrections.
+        
+        :param distance: the maximum edit distance to consider for enumeration
+        :param minSimilarity: threshold for additional fuzzy terms after edits have been exhausted
+        """
+        if field not in self.spellcheckers:
+            self.spellcheckers[field] = SpellChecker(self.terms(field, counts=True))
+        corrections = set()
+        for words in itertools.islice(self.spellcheckers[field].correct(text), distance + 1):
+            for word in words:
+                yield word
+            corrections.update(words)
+        if minSimilarity:
+            words = set(self.terms(field, text, minSimilarity=minSimilarity)) - corrections
+            for word in sorted(words, key=self.spellcheckers[field].__getitem__, reverse=True):
+                yield word
 
 class IndexSearcher(Searcher, lucene.IndexSearcher, IndexReader):
     """Inherited lucene IndexSearcher, with a mixed-in IndexReader.
@@ -461,7 +516,7 @@ class IndexWriter(lucene.IndexWriter):
         :param params: store,index,termvector options compatible with `Field`_
         """
         self.fields[name] = cls(name, **params)
-    def add(self, document=(), **terms):
+    def add(self, document=(), boost=1.0, **terms):
         """Add document to index.
         Document is comprised of name: value pairs, where the values may be one or multiple strings.
         
@@ -470,6 +525,7 @@ class IndexWriter(lucene.IndexWriter):
         """
         terms.update(document)
         doc = lucene.Document()
+        doc.boost = boost
         for name, values in terms.items():
             if isinstance(values, Atomic):
                 values = [values] 
@@ -518,7 +574,10 @@ class Indexer(IndexWriter):
         return iter(self.indexSearcher)
     def __getitem__(self, id):
         return self.indexSearcher[id]
-    def commit(self):
-        "Commit writes and refresh searcher.  Not thread-safe."
+    def commit(self, **caches):
+        """Commit writes and refresh searcher.  Not thread-safe.
+        
+        :param caches: :meth:`Searcher.reopen` caches to refresh
+        """
         IndexWriter.commit(self)
-        self.indexSearcher = self.indexSearcher.reopen()
+        self.indexSearcher = self.indexSearcher.reopen(**caches)
