@@ -3,6 +3,8 @@ Query wrappers and search utilities.
 """
 
 import itertools
+import bisect
+import heapq
 import lucene
 
 class Query(object):
@@ -17,6 +19,12 @@ class Query(object):
         "Return lucene CachingWrapperFilter, optionally just QueryWrapperFilter."
         filter = lucene.QueryWrapperFilter(self)
         return lucene.CachingWrapperFilter(filter) if cache else filter
+    def terms(self):
+        "Generate set of query term items."
+        terms = lucene.HashSet()
+        self.extractTerms(terms)
+        for term in itertools.imap(lucene.Term.cast_, terms):
+            yield term.field(), term.text()
     @classmethod
     def term(cls, name, value):
         "Return lucene TermQuery."
@@ -44,9 +52,10 @@ class Query(object):
         return SpanQuery(lucene.SpanTermQuery, lucene.Term(name, value))
     @classmethod
     def near(cls, name, *values, **kwargs):
-        "Return :meth:`SpanNearQuery <SpanQuery.near>` from terms."
-        queries = (cls.span(name, value) for value in values)
-        return SpanQuery.near(*queries, **kwargs)
+        """Return :meth:`SpanNearQuery <SpanQuery.near>` from terms.
+        Term values which supply another field name will be masked."""
+        spans = (cls.span(name, value) if isinstance(value, basestring) else cls.span(*value).mask(name) for value in values)
+        return SpanQuery.near(*spans, **kwargs)
     @classmethod
     def prefix(cls, name, value):
         "Return lucene PrefixQuery."
@@ -131,10 +140,16 @@ class SpanQuery(Query):
         
         :param slop: default 0
         :param inOrder: default True
+        :param collectPayloads: default True
         """
-        slop = kwargs.pop('slop', 0)
-        inOrder = kwargs.pop('inOrder', True)
-        return SpanQuery(lucene.SpanNearQuery, spans, slop, inOrder, **kwargs)
+        args = map(kwargs.pop, ('slop', 'inOrder'), (0, True))
+        if 'collectPayloads' in kwargs:
+            args.append(kwargs.pop('collectPayloads'))
+        assert not kwargs, 'unexpected keyword arguments: {0}'.format(list(kwargs))
+        return SpanQuery(lucene.SpanNearQuery, spans, *args)
+    def mask(self, name):
+        "Return lucene FieldMaskingSpanQuery, which allows combining span queries from different fields."
+        return SpanQuery(lucene.FieldMaskingSpanQuery, self, name)
 
 class HitCollector(lucene.PythonCollector if hasattr(lucene, 'PythonCollector') else lucene.PythonHitCollector):
     "Collect all ids and scores efficiently."
@@ -216,3 +231,69 @@ class Highlighter(lucene.Highlighter):
                 self.fragmentScorer = lucene.HighlighterSpanScorer(self.query, self.field, tokens)
             return map(unicode, self.getBestTextFragments(tokens, text, True, count))
         return list(self.getBestFragments(self.analyzer, field, text, count))
+
+class SpellChecker(dict):
+    """Correct spellings and suggest words for queries.
+    Supply a vocabulary mapping words to (reverse) sort keys, such as document frequencies.
+    """
+    def __init__(self, *args, **kwargs):
+        dict.__init__(self, *args, **kwargs)
+        self.words = sorted(self)
+        self.alphabet = sorted(set(itertools.chain.from_iterable(self.words)))
+        self.suffix = self.alphabet[-1] * max(itertools.imap(len, self.words)) if self.alphabet else ''
+        self.prefixes = set(word[:stop] for word in self.words for stop in range(len(word) + 1))
+    def suggest(self, prefix, count=None):
+        "Return ordered suggested words for prefix."
+        start = bisect.bisect_left(self.words, prefix)
+        stop = bisect.bisect_right(self.words, prefix + self.suffix, start)
+        words = self.words[start:stop]
+        if count is not None and count < len(words):
+            return heapq.nlargest(count, words, key=self.__getitem__)
+        words.sort(key=self.__getitem__, reverse=True)
+        return words
+    def edits(self, word, length=0):
+        "Return set of potential words one edit distance away, mapped to valid prefix lengths."
+        pairs = [(word[:index], word[index:]) for index in range(len(word) + 1)]
+        deletes = (head + tail[1:] for head, tail in pairs[:-1])
+        transposes = (head + tail[1::-1] + tail[2:] for head, tail in pairs[:-2])
+        edits = {} if length else dict.fromkeys(itertools.chain(deletes, transposes), 0)
+        for head, tail in pairs[length:]:
+            if head not in self.prefixes:
+                break
+            for char in self.alphabet:
+                prefix = head + char
+                if prefix in self.prefixes:
+                    edits[prefix + tail] = edits[prefix + tail[1:]] = len(prefix)
+        return edits
+    def correct(self, word):
+        "Generate ordered sets of words by increasing edit distance."
+        previous, edits = set(), {word: 0}
+        for distance in range(len(word)):
+            yield sorted(itertools.ifilter(self.__contains__, edits), key=self.__getitem__, reverse=True)
+            previous.update(edits)
+            groups = itertools.imap(self.edits, edits, edits.values())
+            edits = dict((edit, group[edit]) for group in groups for edit in group if edit not in previous)
+
+class SpellParser(lucene.PythonQueryParser):
+    """Inherited lucene QueryParser which corrects spelling.
+    Assign a `searcher`_ attribute or override :meth:`correct` implementation.
+    """
+    def correct(self, term):
+        "Return term with text replaced as necessary."
+        field = term.field()
+        for text in self.searcher.correct(field, term.text()):
+            return lucene.Term(field, text)
+        return term
+    def getFieldQuery(self, field, text, *args):
+        "Correct term and phrase queries."
+        query = lucene.PythonQueryParser.getFieldQuery(self, field, text, *args)
+        if args:
+            return query
+        if lucene.TermQuery.instance_(query):
+            term = lucene.TermQuery.cast_(query).term
+            return lucene.TermQuery(self.correct(term))
+        query = lucene.PhraseQuery.cast_(query)
+        phrase = lucene.PhraseQuery()
+        for position, term in zip(query.positions, query.terms):
+            phrase.add(self.correct(term), position)
+        return phrase
