@@ -1,17 +1,18 @@
 """
 Restful json `CherryPy <http://cherrypy.org/>`_ server.
 
-    $ python server.py `directory`,... [-r|--read-only] [-c|--config filename]
-
 CherryPy and Lucene VM integration issues:
- * Autoreload is not compatible with the VM initialization.
- * WorkerThreads must be attached to the VM.
- * Also recommended that the VM ignores keyboard interrupts for clean server shutdown.
+ * Monitors (such as autoreload) are not compatible with the VM unless threads are attached.
+ * WorkerThreads must be also attached to the VM.
+ * Recommended that the VM ignores keyboard interrupts (-Xrs) for clean server shutdown.
 """
 
 import re
 import httplib
 import threading
+import warnings
+import itertools, collections
+import os, optparse
 from contextlib import contextmanager
 try:
     import simplejson as json
@@ -32,7 +33,7 @@ cherrypy.tools.json = cherrypy.Tool('before_finalize', json_tool)
 def allow_tool(methods=['GET', 'HEAD']):
     "Only allow specified methods."
     request = cherrypy.request
-    if request.method not in methods:
+    if request.method not in methods and not isinstance(request.handler, cherrypy.HTTPError):
         cherrypy.response.headers['allow'] = ', '.join(methods)
         message = "The path {0!r} does not allow {1}.".format(request.path_info, request.method)
         raise cherrypy.HTTPError(httplib.METHOD_NOT_ALLOWED, message)
@@ -47,6 +48,21 @@ def attach_thread(id):
     "Attach current cherrypy worker thread to lucene VM."
     lucene.getVMEnv().attachCurrentThread()
 
+class Autoreloader(cherrypy.process.plugins.Autoreloader):
+    "Autoreload monitor compatible with lucene VM."
+    def run(self):
+        lucene.getVMEnv().attachCurrentThread()
+        cherrypy.process.plugins.Autoreloader.run(self)
+
+class Autorefresher(cherrypy.process.plugins.Monitor):
+    "Automatically refresh WebSearcher."
+    def __init__(self, bus, root, frequency):
+        cherrypy.process.plugins.Monitor.__init__(self, bus, self.run, frequency)
+        self.method = getattr(root, 'commit', root.refresh)
+    def run(self):
+        lucene.getVMEnv().attachCurrentThread()
+        self.method()
+
 @contextmanager
 def HTTPError(status, *exceptions):
     "Interpret exceptions as an HTTPError with given status code."
@@ -61,6 +77,8 @@ class WebSearcher(object):
         'tools.gzip.on': True, 'tools.gzip.mime_types': ['text/html', 'text/plain', 'text/x-json']}
     def __init__(self, *directories, **kwargs):
         self.indexer = engine.MultiSearcher(directories, **kwargs) if len(directories) > 1 else engine.IndexSearcher(*directories, **kwargs)
+    def close(self):
+        self.indexer.close()
     def parse(self, q, **options):
         "Return parsed query using q.* parser options."
         options = dict((key.partition('.')[-1], options[key]) for key in options if key.startswith('q.'))
@@ -72,16 +90,19 @@ class WebSearcher(object):
         elif isinstance(field, basestring):
             (field, boost), = fields
         else:
-            field = [name for name, boost in fields]
-        return q and self.indexer.parse(q, field, **options)
+            field = [name for name, boost in fields] or ''
+        for key in set(options) - set(['op', 'version']):
+            with HTTPError(httplib.BAD_REQUEST, ValueError):
+                options[key] = json.loads(options[key])
+        return q and self.indexer.parse(q, field=field, **options)
     @cherrypy.expose
     @cherrypy.tools.allow(methods=['POST'])
-    def refresh(self):
+    def refresh(self, **caches):
         """Reopen searcher.
         
         **POST** /refresh
         """
-        self.indexer = self.indexer.reopen()
+        self.indexer = self.indexer.reopen(**dict.fromkeys(caches, True))
         return len(self.indexer)
     @cherrypy.expose
     def index(self):
@@ -98,7 +119,7 @@ class WebSearcher(object):
             return dict((str(reader.directory()), reader.numDocs()) for reader in self.indexer.sequentialSubReaders)
         return {str(self.indexer): len(self.indexer)}
     @cherrypy.expose
-    def docs(self, id=None, fields='', multifields=''):
+    def docs(self, id=None, fields=None, multifields=''):
         """Return ids or documents.
         
         **GET** /docs
@@ -117,13 +138,16 @@ class WebSearcher(object):
         """
         if id is None:
             return list(self.indexer)
-        with HTTPError(httplib.NOT_FOUND, ValueError, lucene.JavaError):
-            doc = self.indexer[int(id)]
-        fields = dict.fromkeys(filter(None, fields.split(',')))
+        with HTTPError(httplib.NOT_FOUND, ValueError):
+            id = int(id)
+        if fields is not None:
+            fields = dict.fromkeys(filter(None, fields.split(',')))
         multifields = filter(None, multifields.split(','))
-        return doc.dict(*multifields, **fields)
+        with HTTPError(httplib.NOT_FOUND, lucene.JavaError):
+            doc = self.indexer[id] if fields is None else self.indexer.get(id, *itertools.chain(fields, multifields))
+        return doc.dict(*multifields, **(fields or {}))
     @cherrypy.expose
-    def search(self, q=None, count=None, start=0, fields='', multifields='', sort=None, facets='', hl='', mlt=None, **options):
+    def search(self, q=None, count=None, start=0, fields=None, multifields='', sort=None, facets='', hl='', mlt=None, spellcheck=0, **options):
         """Run query and return documents.
         
         **GET** /search?
@@ -145,7 +169,7 @@ class WebSearcher(object):
                 field name, optional type, minus sign indicates descending
             
             &facets=\ *chars*,...
-                include facet counts for given field names
+                include facet counts for given field names;  facets filters are cached
             
             &hl=\ *chars*,... &hl.count=1&hl.tag=strong&hl.enable=[fields|terms]
                 | stored fields to return highlighted
@@ -157,21 +181,24 @@ class WebSearcher(object):
                 | optional document fields to match
                 | optional MoreLikeThis settings: mlt.minTermFreq, mlt.minDocFreq,...
             
+            &spellcheck=\ *int*
+                | maximum number of spelling corrections to return for each query term, grouped by field
+                | original query is still run;  use q.spellcheck=true to affect query parsing
+            
             :return:
                 | {
                 | "query": *string*,
                 | "count": *int*,
                 | "docs": [{"__id__": *int*, "__score__": *number*, "__highlights__": {*string*: *array*,... }, *string*: *string*\|\ *array*,... },... ],
-                | "facets": {*chars*: {*chars*: *int*,... },... },
+                | "facets": {*string*: {*string*: *int*,... },... },
+                | "spellcheck": {*string*: {*string*: [*string*,... ],... },... },
                 | }
         """
         with HTTPError(httplib.BAD_REQUEST, ValueError):
             start = int(start)
-        if count is not None:
-            with HTTPError(httplib.BAD_REQUEST, ValueError):
+            if count is not None:
                 count = int(count) + start
-        fields = dict.fromkeys(filter(None, fields.split(',')))
-        multifields = filter(None, multifields.split(','))
+            spellcheck = int(spellcheck)
         reverse = False
         searcher = getattr(self.indexer, 'indexSearcher', self.indexer)
         if sort is not None:
@@ -194,8 +221,11 @@ class WebSearcher(object):
             with HTTPError(httplib.BAD_REQUEST, ValueError):
                 attrs = dict((key.partition('.')[-1], json.loads(options[key])) for key in options if key.startswith('mlt.'))
             q = searcher.morelikethis(mlt, *mltfields, **attrs)
-        hits = searcher.search(q, count=count, sort=sort, reverse=reverse)
-        result = {'query': unicode(q), 'count': hits.count, 'docs': []}
+        if count == 0:
+            hits = searcher.search(q, count=1, sort=sort, reverse=reverse)[:0]
+        else:
+            hits = searcher.search(q, count=count, sort=sort, reverse=reverse)
+        result = {'query': q and unicode(q), 'count': hits.count, 'docs': []}
         tag = options.get('hl.tag', 'strong')
         field = 'fields' not in options.get('hl.enable', '') or None
         span = 'terms' not in options.get('hl.enable', '')
@@ -203,6 +233,11 @@ class WebSearcher(object):
             hl = dict((name, searcher.highlighter(q, span=span, field=(field and name), formatter=tag)) for name in hl.split(','))
         with HTTPError(httplib.BAD_REQUEST, ValueError):
             count = int(options.get('hl.count', 1))
+        if fields is not None:
+            fields = dict.fromkeys(filter(None, fields.split(',')))
+            hits.fields = lucene.MapFieldSelector(list(itertools.chain(fields, multifields, hl)))
+        fields = fields or {}
+        multifields = filter(None, multifields.split(','))
         for hit in hits[start:]:
             doc = hit.dict(*multifields, **fields)
             result['docs'].append(doc)
@@ -210,6 +245,10 @@ class WebSearcher(object):
                 doc['__highlights__'] = dict((name, hl[name].fragments(hit[name], count)) for name in hl if name in hit)
         if facets:
             result['facets'] = searcher.facets(engine.Query.filter.im_func(q), *facets.split(','))
+        if spellcheck:
+            terms = result['spellcheck'] = collections.defaultdict(dict)
+            for name, value in engine.Query.terms.im_func(q):
+                terms[name][value] = list(itertools.islice(searcher.correct(name, value), spellcheck))
         return result
     @cherrypy.expose
     def terms(self, name='', value=':', docs='', counts='', **options):
@@ -227,11 +266,17 @@ class WebSearcher(object):
             
             :return: [*string*,... ]
         
-        **GET** /terms/*chars*/*chars*\[\*\|:*chars*\|~\ *float*\]
+        **GET** /terms/*chars*/*chars*\[\*\|?\|:*chars*\|~\ *float*\]
             Return term values (wildcards, slices, or fuzzy terms) for given field name.
             
             :return: [*string*,... ]
         
+        **GET** /terms/*chars*/*chars*\[\*\|~\]?count=\ *int*
+            Return spellchecked term values ordered by decreasing document frequency.
+            Prefixes (*) are optimized to be suitable for real-time query suggestions;  all terms are cached.
+            
+            :return: [*string*,... ]
+        
         **GET** /terms/*chars*/*chars*
             Return document count with given term.
             
@@ -257,7 +302,13 @@ class WebSearcher(object):
         if ':' in value:
             start, stop = value.split(':')
             return list(self.indexer.terms(name, start, stop or None))
-        if '*' in value:
+        if 'count' in options:
+            count = int(options['count'])
+            if value.endswith('*'):
+                return self.indexer.suggest(name, value.rstrip('*'), count)
+            if value.endswith('~'):
+                return list(itertools.islice(self.indexer.correct(name, value.rstrip('~')), count))
+        if '*' in value or '?' in value:
             return list(self.indexer.terms(name, value))
         if '~' in value:
             value, similarity = value.split('~')
@@ -280,21 +331,21 @@ class WebIndexer(WebSearcher):
         self.lock = threading.Lock()
     @cherrypy.expose
     @cherrypy.tools.allow(methods=['POST'])
-    def refresh(self):
+    def refresh(self, **caches):
         raise cherrypy.HTTPRedirect('/commit', httplib.MOVED_PERMANENTLY)
     @cherrypy.expose
     @cherrypy.tools.allow(methods=['POST'])
-    def commit(self):
+    def commit(self, **caches):
         """Commit write operations.
         
         **POST** /commit
         """
         with self.lock:
-            self.indexer.commit()
+            self.indexer.commit(**dict.fromkeys(caches, True))
         return len(self.indexer)
     @cherrypy.expose
     @cherrypy.tools.allow(methods=['GET', 'HEAD', 'POST'])
-    def docs(self, id=None, fields='', multifields='', docs='[]'):
+    def docs(self, id=None, fields=None, multifields='', docs='[]'):
         """Add or return documents.  See :meth:`WebSearcher.docs` for GET method.
         
         **POST** /docs
@@ -350,29 +401,41 @@ class WebIndexer(WebSearcher):
             field = self.indexer.fields[name]
         return dict((name, str(getattr(field, name))) for name in ['store', 'index', 'termvector'])
 
-def main(root, path='', config=None):
-    "Attach root and run server."
+def start(root, path='', config=None, pidfile='', daemonize=False, autoreload=0, autorefresh=0):
+    """Attach root, subscribe to plugins, and start server.
+    See cherrypy.quickstart and command-line options for documentation."""
     cherrypy.engine.subscribe('start_thread', attach_thread)
-    cherrypy.engine.subscribe('stop', root.indexer.close)
+    cherrypy.engine.subscribe('stop', root.close)
     cherrypy.config['engine.autoreload.on'] = False
+    if pidfile:
+        cherrypy.process.plugins.PIDFile(cherrypy.engine, os.path.abspath(pidfile)).subscribe()
+    if daemonize:
+        cherrypy.config['log.screen'] = False
+        cherrypy.process.plugins.Daemonizer(cherrypy.engine).subscribe()
+    if autoreload:
+        Autoreloader(cherrypy.engine, autoreload).subscribe()
+    if autorefresh:
+        Autorefresher(cherrypy.engine, root, autorefresh).subscribe()
     cherrypy.quickstart(root, path, config)
 
+def main(root, path='', config=None):
+    warnings.warn("'main' function has been renamed 'start'.", DeprecationWarning)
+    start(root, path, config)
+
+parser = optparse.OptionParser(usage='python %prog [index_directory ...]')
+parser.add_option('-r', '--read-only', action='store_true', help='expose only read methods; no write lock')
+parser.add_option('-c', '--config', help='optional configuration file or json object of global params')
+parser.add_option('-p', '--pidfile', metavar='FILE', help='store the process id in the given file')
+parser.add_option('-d', '--daemonize', action='store_true', help='run the server as a daemon')
+parser.add_option('--autoreload', type=int, metavar='SECONDS', help='automatically reload modules; replacement for engine.autoreload')
+parser.add_option('--autorefresh', type=int, metavar='SECONDS', help='automatically refresh index')
+
 if __name__ == '__main__':
-    import os, optparse
-    parser = optparse.OptionParser(usage='python %prog [index_directory ...]')
-    parser.add_option('-r', '--read-only', action='store_true', dest='read', help='expose only read methods; no write lock')
-    parser.add_option('-c', '--config', dest='config', help='optional configuration file or json object of global params')
-    parser.add_option('-p', '--pidfile', dest='pidfile', help='store the process id in the given file')
-    parser.add_option('-d', '--daemonize', action='store_true', dest='daemonize', help='run the server as a daemon')
     options, args = parser.parse_args()
     if lucene.getVMEnv() is None:
         lucene.initVM(lucene.CLASSPATH, vmargs='-Xrs')
-    root = (WebSearcher if (options.read or len(args) > 1) else WebIndexer)(*args)
+    read_only = options.__dict__.pop('read_only')
+    root = (WebSearcher if (read_only or len(args) > 1) else WebIndexer)(*args)
     if options.config and not os.path.exists(options.config):
         options.config = {'global': json.loads(options.config)}
-    if options.pidfile:
-        cherrypy.process.plugins.PIDFile(cherrypy.engine, os.path.abspath(options.pidfile)).subscribe()
-    if options.daemonize:
-        cherrypy.config['log.screen'] = False
-        cherrypy.process.plugins.Daemonizer(cherrypy.engine).subscribe() 
-    main(root, config=options.config)
+    start(root, **options.__dict__)
