@@ -1,5 +1,5 @@
 import unittest
-import os, optparse
+import os
 import tempfile, shutil
 import itertools
 import collections
@@ -11,9 +11,6 @@ from lupyne import engine
 import fixture
 
 numeric = hasattr(lucene, 'NumericField')
-parser = optparse.OptionParser()
-parser.add_option("-v", "--verbose", action="store_true", dest="verbose")
-options, args = parser.parse_args()
 
 if issubclass(lucene.TokenFilter, collections.Iterable):
     def typeAsPayload(tokens):
@@ -32,7 +29,6 @@ else:
 class BaseTest(unittest.TestCase):
     def setUp(self):
         self.tempdir = tempfile.mkdtemp(dir=os.path.dirname(__file__))
-        self.verbose = options.verbose
     def tearDown(self):
         shutil.rmtree(self.tempdir)
 
@@ -82,6 +78,7 @@ class TestCase(BaseTest):
         doc['name'] == 'sample' and doc['tag'] in ('python', 'search')
         self.assertRaises(KeyError, doc.__getitem__, 'key')
         assert doc.getlist('name') == ['sample'] and doc.getlist('key') == []
+        assert indexer.get(0, 'name').dict() == {'name': 'sample'}
         assert indexer.count('text', 'hello') == indexer.count('text:hello') == 1
         assert sorted(indexer.names()) == ['name', 'tag', 'text']
         assert sorted(indexer.names('indexed')) == ['tag', 'text']
@@ -106,6 +103,19 @@ class TestCase(BaseTest):
         assert not indexer.search('hello') and indexer.search('hello', field='text')
         assert indexer.search('text:hello hi') and not indexer.search('text:hello hi', op='and')
         assert indexer.search('text:*hello', allowLeadingWildcard=True)
+        query = engine.Query.span('text', 'world')
+        self.assertRaises(AssertionError, query.near, option=None)
+        if lucene.VERSION >= '2.9':
+            assert str(query.mask('name')) == 'mask(text:world) as name'
+            query = engine.Query.near('text', 'hello', ('tag', 'python'), slop=-1, inOrder=False)
+            assert str(query) == 'spanNear([text:hello, mask(tag:python) as text], -1, false)' and indexer.count(query) == 1
+            query = engine.Query.near('text', 'hello', 'world')
+            (doc, items), = indexer.spans(query, payloads=True)
+            (start, stop, payloads), = items
+            assert doc == 0 and start == 0 and stop == 2 and payloads == ['<ALPHANUM>', '<ALPHANUM>']
+            query = engine.Query.near('text', 'hello', 'world', collectPayloads=False)
+            (doc, items), = indexer.spans(query, payloads=True)
+            assert doc == 0 and items == []
         indexer.delete('name:sample')
         indexer.delete('tag', 'python')
         assert 0 in indexer
@@ -133,8 +143,15 @@ class TestCase(BaseTest):
         for name, params in fixture.constitution.fields.items():
             indexer.set(name, **params)
         for doc in fixture.constitution.docs():
-            indexer.add(doc)
+            indexer.add(doc, boost=('article' in doc) + 1.0)
         indexer.commit()
+        assert indexer.filters == indexer.spellcheckers == {}
+        assert indexer.facets([], 'amendment') and indexer.suggest('amendment', '')
+        assert list(indexer.filters) == list(indexer.spellcheckers) == ['amendment']
+        indexer.delete('amendment', doc['amendment'])
+        indexer.add(doc)
+        indexer.commit(filters=True, spellcheckers=True)
+        assert list(indexer.filters) == list(indexer.spellcheckers) == ['amendment']
         searcher = engine.MultiSearcher([indexer.directory, self.tempdir])
         assert searcher.count() == len(searcher) == 2 * len(indexer)
         searcher = searcher.reopen()
@@ -162,6 +179,7 @@ class TestCase(BaseTest):
         assert hit['article'] == 'Preamble'
         assert sorted(hit.dict()) == ['__id__', '__score__', 'article']
         hits = indexer.search('people', field='text')
+        assert hits[0]['article'] == 'Preamble'
         assert len(hits) == hits.count == 8
         assert set(map(type, hits.ids)) == set([int]) and set(map(type, hits.scores)) == set([float])
         ids = hits.ids
@@ -221,11 +239,12 @@ class TestCase(BaseTest):
         assert str(-query) == '-text:persons'
         query = +query
         query -= engine.Query.term('text', 'papers')
+        assert set(query.terms()) == set([('text', 'persons'), ('text', 'papers')])
         assert str(query[-1]) == '-text:papers'
         assert len(query) == len(list(query)) == 2
         span = engine.Query.span('text', 'persons')
         count = indexer.count(span)
-        near = engine.Query.near('text', 'persons', 'papers', slop=1)
+        near = engine.Query.near('text', 'persons', 'papers', slop=1, inOrder=False)
         assert indexer.count(span - near) == count
         near = span.near(engine.Query.span('text', 'papers') | engine.Query.span('text', 'things'), slop=1)
         assert indexer.count(span - near) == count - 1
@@ -250,6 +269,23 @@ class TestCase(BaseTest):
         assert str(indexer.morelikethis('jury', 'text', minDocFreq=4, minTermFreq=1)) == 'text:jury'
         assert str(indexer.morelikethis('jury', 'article')) == ''
         self.assertRaises(lucene.JavaError, indexer.morelikethis, 'jury')
+        assert indexer.suggest('missing', '') == list(indexer.correct('missing', '')) == []
+        assert indexer.suggest('text', '')[:8] == ['shall', 'states', 'any', 'have', 'united', 'congress', 'state', 'constitution']
+        assert indexer.suggest('text', 'con')[:2] == ['congress', 'constitution']
+        assert indexer.suggest('text', 'congress') == ['congress']
+        assert indexer.suggest('text', 'congresses') == []
+        assert list(indexer.correct('text', 'writ', distance=0, minSimilarity=None)) == ['writ']
+        assert list(indexer.correct('text', 'write', distance=0, minSimilarity=None)) == []
+        assert list(indexer.correct('text', 'write', distance=0)) == ['crime', 'writs', 'written', 'writ']
+        assert list(indexer.correct('text', 'write', distance=0, minSimilarity=0.7)) == ['writs', 'writ']
+        assert list(indexer.correct('text', 'write', distance=1, minSimilarity=None)) == ['writs', 'writ']
+        assert list(indexer.correct('text', 'write', distance=1)) == ['writs', 'writ', 'crime', 'written']
+        assert list(indexer.correct('text', 'write', distance=1, minSimilarity=0.7)) == ['writs', 'writ']
+        assert list(indexer.correct('text', 'write', minSimilarity=0.9)) == ['writs', 'writ', 'crime', 'written']
+        query = indexer.parse('text:write', spellcheck=True)
+        assert lucene.TermQuery.instance_(query) and str(query) == 'text:writs'
+        query = indexer.parse('"hello world"', field='text', spellcheck=True)
+        assert lucene.PhraseQuery.instance_(query) and str(query) == 'text:"held would"'
         del indexer
         assert engine.Indexer(self.tempdir)
     
@@ -350,9 +386,9 @@ class TestCase(BaseTest):
     def testFields(self):
         "Custom field tests."
         indexer = engine.Indexer(self.tempdir)
-        indexer.set('amendment', engine.FormatField, format='{0:02n}', store=True)
+        indexer.set('amendment', engine.FormatField, format='{0:02d}', store=True)
         indexer.set('date', engine.DateTimeField, store=True)
-        indexer.set('size', engine.FormatField, format='{0:04n}', store=True)
+        indexer.set('size', engine.FormatField, format='{0:04d}', store=True)
         for doc in fixture.constitution.docs():
             if 'amendment' in doc:
                 indexer.add(amendment=int(doc['amendment']), date=doc['date'], size=len(doc['text']))
