@@ -1,12 +1,12 @@
 import unittest
-import sys
+import os, sys
 import subprocess
 import operator
 import httplib
 import socket, errno
 from contextlib import contextmanager
 import cherrypy
-from lupyne import client
+from lupyne import client, server
 import fixture, local
 
 @contextmanager
@@ -23,18 +23,24 @@ class BaseTest(local.BaseTest):
     ports = 8080, 8081
     def setUp(self):
         local.BaseTest.setUp(self)
-        self.servers = [self.start(self.ports[0], self.tempdir)]
-        self.servers += [self.start(port, self.tempdir, self.tempdir) for port in self.ports[1:]] # concurrent searchers
+        pidfile = os.path.join(self.tempdir, 'pid')
+        self.servers = (
+            self.start(self.ports[0], self.tempdir, '--autoreload=1'),
+            self.start(self.ports[1], self.tempdir, self.tempdir, '-p', pidfile), # concurrent searchers
+        )
+        assert int(open(pidfile).read()) == self.servers[-1].pid
+    def run(self, result):
+        self.stderr = None if result.showAll else subprocess.PIPE
+        local.BaseTest.run(self, result)
     def tearDown(self):
         for server in self.servers:
             self.stop(server)
         local.BaseTest.tearDown(self)
     def start(self, port, *args):
         "Start server in separate process on given port."
-        params = sys.executable, '-m', 'lupyne.server', '-c', '{{"server.socket_port": {0:n}}}'.format(port)
-        stderr = None if self.verbose else subprocess.PIPE
+        params = sys.executable, '-m', 'lupyne.server', '-c', '{{"server.socket_port": {0:d}}}'.format(port)
         cherrypy.process.servers.wait_for_free_port('localhost', port)
-        server = subprocess.Popen(params + args, stderr=stderr)
+        server = subprocess.Popen(params + args, stderr=self.stderr)
         cherrypy.process.servers.wait_for_occupied_port('localhost', port)
         assert server.poll() is None
         return server
@@ -95,6 +101,7 @@ class TestCase(BaseTest):
         assert resource.get('/docs/0') == {'name': 'sample'}
         assert resource.get('/docs/0', fields='missing') == {'missing': None}
         assert resource.get('/docs/0', multifields='name') == {'name': ['sample']}
+        assert resource.get('/docs/0', fields='', multifields='missing') == {'missing': []}
         assert resource.get('/terms') == ['name', 'text']
         assert resource.get('/terms', option='unindexed') == []
         assert resource.get('/terms/text') == ['hello', 'world']
@@ -113,6 +120,8 @@ class TestCase(BaseTest):
             resource.get('/search', sort='x,y')
         with assertRaises(httplib.HTTPException, httplib.BAD_REQUEST):
             resource.get('/search', count=1, sort='x:str')
+        assert resource.get('/search', count=0) == {'count': 1, 'query': None, 'docs': []}
+        assert resource.get('/search', fields='')['docs'] == [{'__id__': 0, '__score__': 1.0}]
         assert sorted(hits) == ['count', 'docs', 'query']
         assert hits['count'] == 1
         doc, = hits['docs']
@@ -133,27 +142,31 @@ class TestCase(BaseTest):
         resource = client.Resource('localhost', self.ports[-1])
         assert resource.get('/docs') == []
         try:
-            assert resource.post('/refresh') == 2
+            assert resource.post('/refresh', filters=True) == 2
         except httplib.HTTPException as (status, reason, body): # unsupported in lucene 2.4
             assert body['traceback'].splitlines()[-1] == "AttributeError: 'IndexReader' object has no attribute 'sequentialSubReaders'"
         else:
             assert resource.get('/docs') == [0, 1]
         with assertRaises(httplib.HTTPException, httplib.NOT_FOUND):
             resource.get('/fields')
+        with assertRaises(httplib.HTTPException, httplib.NOT_FOUND):
+            resource.post('/commit')
         resource = client.Resource('localhost', self.ports[0])
         assert not resource.delete('/search', q='name:sample')
         assert resource.get('/docs') == [0]
         assert not resource.post('/commit')
         assert resource.get('/docs') == []
         with assertRaises(httplib.HTTPException, httplib.MOVED_PERMANENTLY):
-            resource.post('/refresh')
+            resource.post('/refresh', spellcheckers=True)
         resource = client.Resource('localhost', self.ports[-1] + 1)
         with assertRaises(socket.error, errno.ECONNREFUSED):
             resource.get('/')
+        config = {'global': {'server.socket_port': self.ports[0], 'log.screen': not self.stderr}}
+        self.assertRaises(IOError, server.start, server.WebIndexer(), config=config, autoreload=1, autorefresh=1)
     
     def testBasic(self):
         "Remote text indexing and searching."
-        resource = client.Resource('localhost', str(self.ports[0]))
+        resource = client.Resource('localhost', self.ports[0])
         assert resource.get('/fields') == []
         for name, settings in fixture.constitution.fields.items():
             assert resource.put('/fields/' + name, **settings)
@@ -163,7 +176,7 @@ class TestCase(BaseTest):
             assert sorted(resource.get('/fields/' + name)) == ['index', 'store', 'termvector']
         resource.post('/docs/', docs=list(fixture.constitution.docs()))
         assert resource.get('/').values() == [35]
-        resource.post('/commit')
+        resource.post('/commit', spellcheckers=True, filters='')
         assert resource.get('/terms') == ['amendment', 'article', 'date', 'text']
         articles = resource.get('/terms/article')
         articles.remove('Preamble')
@@ -172,15 +185,31 @@ class TestCase(BaseTest):
         assert resource.get('/terms/text/:0') == []
         assert resource.get('/terms/text/z:') == []
         assert resource.get('/terms/text/right:right~') == resource.get('/terms/text/right*') == ['right', 'rights']
+        assert resource.get('/terms/text/writ%3f') == ['writs']
+        assert resource.get('/terms/text/writ*') == ['writ', 'writing', 'writings', 'writs', 'written']
+        assert resource.get('/terms/text/*?count=0') == []
+        assert resource.get('/terms/text/writ*?count=10') == ['writs', 'writ', 'writing', 'writings', 'written']
+        assert resource.get('/terms/text/writ*?count=3') == ['writs', 'writ', 'writing']
         assert resource.get('/terms/text/right~') == resource.get('/terms/text/right~0.5') == ['eight', 'right', 'rights']
+        assert resource.get('/terms/text/right~?count=3') == ['right', 'eight', 'rights']
+        assert resource.get('/terms/text/right~?count=5') == ['right', 'eight', 'rights', 'high']
+        assert resource.get('/terms/text/write~?count=5') == ['writs', 'writ', 'crime', 'written']
         docs = resource.get('/terms/text/people/docs')
         assert resource.get('/terms/text/people') == len(docs) == 8
         counts = dict(resource.get('/terms/text/people/docs/counts'))
         assert sorted(counts) == docs and all(counts.values()) and sum(counts.values()) > len(counts)
         positions = dict(resource.get('/terms/text/people/docs/positions'))
         assert sorted(positions) == docs and map(len, positions.values()) == counts.values()
-        result = resource.get('/search', q='text:"We the People"')
+        result = resource.get('/search', **{'q.field': 'text', 'q': 'write "hello world"', 'spellcheck': 3})
+        terms = result['spellcheck'].pop('text')
+        assert result['docs'] == [] and result['spellcheck'] == {}
+        assert terms == {'write': ['writs', 'writ', 'crime'], 'world': ['would', 'hold', 'gold'], 'hello': ['held', 'well']}
+        result = resource.get('/search', **{'q.field': 'text', 'q': 'write "hello world"', 'q.spellcheck': 'true'})
+        assert result['query'] == 'text:writs text:"held would"'
+        assert result['count'] == len(result['docs']) == resource.get('/terms/text/writs') == 2
+        result = resource.get('/search', q='text:"We the People"', **{'q.phraseSlop': 3})
         assert sorted(result) == ['count', 'docs', 'query'] and result['count'] == 1
+        assert result['query'] in ('text:"we ? people"~3', 'text:"we people"~3') # second query is 2.4 analysis
         doc, = result['docs']
         assert sorted(doc) == ['__id__', '__score__', 'article']
         assert doc['article'] == 'Preamble' and doc['__id__'] >= 0 and 0 < doc['__score__'] < 1
@@ -207,7 +236,7 @@ class TestCase(BaseTest):
         assert result['count'] == 1
         doc, = result['docs']
         assert doc['amendment'] == '1'
-        doc, = resource.get('/search', q='amendment:1', hl='amendment')['docs']
+        doc, = resource.get('/search', q='amendment:1', hl='amendment', fields='article')['docs']
         assert doc['__highlights__'] == {'amendment': ['<strong>1</strong>']}
         doc, = resource.get('/search', q='amendment:1', hl='amendment,article', **{'hl.count': 2, 'hl.tag': 'em'})['docs']
         assert doc['__highlights__'] == {'amendment': ['<em>1</em>']}
